package com.developgadget.unity_ads;

import android.app.Activity;

import androidx.annotation.NonNull;

import com.unity3d.ads.UnityAds;

import java.util.Map;

import io.flutter.Log;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

public class UnityAdsPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
    private static MethodChannel Channel;
    private static UnityAdsPlugin Instance;
    private static Activity InstanceActivity;
    final Listeners ListenersAds = new Listeners();

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        this.UnityAttach(flutterPluginBinding.getBinaryMessenger());
    }

    public static UnityAdsPlugin getInstance() {
        return Instance;
    }

    public static void registerWith(Registrar registrar) {
        if (InstanceActivity == null) InstanceActivity = registrar.activity();
        if (Instance == null) Instance = new UnityAdsPlugin();
        Instance.UnityAttach(registrar.messenger());
    }

    private void UnityAttach(BinaryMessenger messenger) {
        if (UnityAdsPlugin.Instance == null)
            UnityAdsPlugin.Instance = new UnityAdsPlugin();
        if (UnityAdsPlugin.Channel != null)
            return;
        UnityAdsPlugin.Channel = new MethodChannel(messenger, "UnityAds");
        UnityAdsPlugin.Channel.setMethodCallHandler(this);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        try {
            switch (call.method) {
                case "init":
                    UnityAds.initialize(this.InstanceActivity,
                            call.<String>argument("gameId"), call.<Boolean>argument("testMode"));
                    UnityAds.addListener(this.ListenersAds);
                    break;
                case "getDebugMode":
                    result.success(UnityAds.getDebugMode());
                    break;
                case "getDefaultPlacementState":
                    result.success(UnityAds.getPlacementState().ordinal());
                    break;
                case "getPlacementState":
                    result.success(UnityAds.getPlacementState(call.<String>argument("placementId")).ordinal());
                    break;
                case "getVersion":
                    result.success(UnityAds.getVersion());
                    break;
                case "isInitialized":
                    result.success(UnityAds.isInitialized());
                    break;
                case "isReady":
                    result.success(UnityAds.isReady(call.<String>argument("placementId")));
                    break;
                case "isSupported":
                    result.success(UnityAds.isSupported());
                    break;
                case "setDebugMode":
                    UnityAds.setDebugMode(call.<Boolean>argument("debugMode"));
                    break;
                case "show":
                    if (call.hasArgument("placementId")) {
                        UnityAds.show(InstanceActivity, call.<String>argument("placementId"));
                    } else {
                        UnityAds.show(InstanceActivity);
                    }
                    break;
            }
            result.success(Boolean.TRUE);
        } catch (Exception e) {
            Log.e("UnityAds", "Error " + e.toString());
        }
    }

    void OnMethodCallHandler(final String method, final Map<String, Object> arguments) {
        try {
            UnityAdsPlugin.InstanceActivity.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Channel.invokeMethod(method, arguments);
                }
            });
        } catch (Exception e) {
            Log.e("UnityAds", "Error " + e.toString());
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    }

    @Override
    public void onAttachedToActivity(ActivityPluginBinding binding) {
        UnityAdsPlugin.InstanceActivity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {
        UnityAdsPlugin.InstanceActivity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivity() {

    }
}
